#ifndef APR_POOL_HH
#define APR_POOL_HH

struct apr_pool_t;

namespace apr {

class Pool {
    protected:
        enum DoNothing { doNothing };

        /**
         * Do-nothing constructor that Svn::Pool::Pool() uses.
         * Otherwise, the <tt>apr_pool_create</tt> and
         * <tt>svn_pool_create</tt> would be redundant.
         **/
        Pool(DoNothing) : me(0), owner(true) {}

    public:

        /** Creates a new top-level pool. **/
        Pool();

        /** Wraps a C reference to an existing pool. **/
        explicit Pool(apr_pool_t *pool) : me(pool), owner(false) {}

        /** Creates a subpool of an existing pool. **/
        Pool(Pool &pool);

        /** Destroys the pool. **/
        virtual ~Pool();

        /**
         * Clears all memory in the pool and runs all the cleanups.
         * This also destroys all subpools.
         * @remark This does not actually free the memory. It just allows the
         *         pool to re-use this memory for the next allocation.
         **/
        virtual void clear();

        /**
         * Returns a handle for accessing the pool.
         * This allows an apr::Pool to be passed whenever an apr_pool_t* is
         * expected.
         **/
        operator apr_pool_t *const () const;

    protected:
        apr_pool_t *me;
        bool owner;

        /** Noncopyable. **/
        Pool& operator=(const Pool&);
};

}

#endif
