#include <svn/exception.hh>
#include <stdlib.h>

using namespace std;
using namespace boost;

namespace svn {

Exception::Exception(svn_error_t *cstyle) : aprErr(cstyle->apr_err),
        srcErr(cstyle->src_err), message(cstyle->message), pool(cstyle->pool),
        line(cstyle->line) {
    if (cstyle->child != SVN_NO_ERROR) {
        child.reset(new Exception(cstyle->child));

        // Null this so recursive svn_error_clear doesn't die
        cstyle->child = NULL;
    }
    if (cstyle->file != NULL)
        file = cstyle->file;

    svn_error_clear(cstyle);
}

Exception::Exception(const Exception &o) : aprErr(o.aprErr), srcErr(o.srcErr),
        message(o.message), pool((apr_pool_t*) o.pool),
        file(o.file), line(o.line) {
    if (o.child.get() != NULL) {
        child.reset(new Exception(*o.child));
    }
}

void Exception::handle(svn_error_t *cstyle) {
    if (cstyle != SVN_NO_ERROR) {
        throw Exception(cstyle);
    }
}

svn_error_t *Exception::toCStyle() {
    svn_error_t *cstyle = svn_error_create(aprErr, srcErr, child->toCStyle(),
            message.c_str());
    cstyle->file = file.c_str();
    cstyle->line = line;
    return cstyle;
}

const char *Exception::what() const throw() {
    return message.c_str();
}

}
