#ifndef SVN_EXCEPTION_HH
#define SVN_EXCEPTION_HH

#include <string>
#include <exception>
#include <boost/shared_ptr.hpp>
#include <svn/pool.hh>
#include <svn_error.h>

namespace svn {

class Exception : public std::exception {
    public:
        /**
         * Creates a C++-style error chain from a C one and destroys the C one.
         **/
        explicit Exception(svn_error_t*);

        /** Creates a deep copy of another C++-style exception. **/
        Exception(const Exception&);

        /**
         * Throws an error if parameter is not <tt>SVN_NO_ERROR</tt>.
         * Useful for wrapping around functions to the C API.
         **/
        static void handle(svn_error_t*);

        virtual ~Exception() throw() {}

        virtual const char *what() const throw();

        /** Creates a C-style error chain in the global error pool. **/
        svn_error_t *toCStyle();
    private:
        apr_status_t aprErr;
        int srcErr;
        boost::shared_ptr<Exception> child;
        std::string message;
        Pool pool;
        std::string file;
        long line;
};

}

#endif //!SVN_EXCEPTION_HH
